@@ -18,6 +18,7 @@ from .dehardsub import hardsub_mask
 from .progress import Progress, BarColumn, FPSColumn, TextColumn, TimeRemainingColumn
 from .render import clip_async_render
 from .util import get_prop
+from .misc import get_matrix
 
 core = vs.core
 
@@ -346,8 +347,6 @@ def compare(clip_a: vs.VideoNode, clip_b: vs.VideoNode,
 
     Alias for this function is `lvsfunc.comp`.
 
-    Dependencies: mvsfunc
-
     :param clip_a:         Clip to compare
     :param clip_b:         Second clip to compare
     :param frames:         List of frames to compare (Default: ``None``)
@@ -358,23 +357,9 @@ def compare(clip_a: vs.VideoNode, clip_b: vs.VideoNode,
 
     :return:               Interleaved clip containing specified frames from `clip_a` and `clip_b`
     """
-    def _GetMatrix(clip: vs.VideoNode) -> int:
-        frame = clip.get_frame(0)
-        w, h = frame.width, frame.height
-
-        if frame.format.color_family == vs.RGB:
-            return 0
-        if frame.format.color_family == vs.YCOCG:
-            return 8
-        if w <= 1024 and h <= 576:
-            return 5
-        if w <= 2048 and h <= 1536:
-            return 1
-        return 9
-
     def _resample(clip: vs.VideoNode) -> vs.VideoNode:
         # Resampling to 8 bit and RGB to properly display how it appears on your screen
-        return core.resize.Bicubic(clip, format=vs.RGB24, matrix_in=_GetMatrix(clip),
+        return core.resize.Bicubic(clip, format=vs.RGB24, matrix_in=get_matrix(clip),
                                    prefer_props=True, dither_type='error_diffusion')
 
     # Error handling
